<?php

namespace App\Sets\Validation;

use App\Http\Validation\ValidatorBase as Validator;
use App\Sets\Validation\AttachmentValidator;
use App\Sets\Validation\PromotionValidator;

class SetsValidator extends Validator
{
    protected $validator = [
        'title'         =>  'required|chinese_length:2,15',
        'cover'         =>  'required',
        'price'         =>  'required|integer|min:5|max:10000',
        'photographers_count'  =>  'required|integer',
        'makeup_count'         => 'required|integer',
        'logistics_count'      => 'required|integer',
        'join_count'           => 'required|integer',
        'duration'       => 'required|integer',
        'negative_count' => 'required|integer',
        'modification_count'   => 'required|integer',
        'modification_price'   => 'required|integer',
        'modeling_count'       => 'required|integer',
        'deadline_of_day'      => 'required|integer',
        'n_cost'               => 'required|min:1',
        'start_time'           => 'required',
        'end_time'             => 'required',

    ];

    protected $message  = [
        'title.required'            =>  '请填写套系名称',
        'title.chinese_length'      =>  '套系名称长度需在2到15字之间',
        'cover.required'            =>  '请选择封面',
        'price.required'            =>  '请填写套系价格',
        'price.integer'             =>  '套系价格为数字',
        'price.min'                 =>  '价格须在5元以上，100,000以下',
        'price.max'                 =>  '价格须在5元以上，100,000以下',
        'photographers_count.required'  =>  '您还未填写人员配备人数',
        'photographers_count.integer'   =>  '配备人数必须是数字',
        'makeup_count.required'         =>  '您还未填写化妆人员人数',
        'makeup_count.integer'          =>  '化妆人员人数必须是数字',
        'logistics_count.required'      =>  '您还未填写后勤人数',
        'logistics_count.integer'       =>  '后勤人数必须是数字',
        'join_count.required'       =>  '您还未填写最多拍摄人数',
        'join_count.integer'        =>  '拍摄人数必须是数字',
        'duration.required'         =>  '您还未填写最多拍摄时长',
        'duration.integer'          =>  '拍摄时长必须是数字',
        'negative_count.required'   =>  '您还未填写底片数量',
        'negative_count.integer'    =>  '底片数量必须是数字',
        'modification_count.required' => '您还未填写精修片数量',
        'modification_count.integer'  => '精修片数量必须是数字',
        'modification_price.required' => '您还未填写额外精修价格',
        'modification_price.integer'  => '额外精修价格必须是数字',
        'modeling_count.required'     => '您还未填写造型数量',
        'modeling_count.integer'      => '造型数量必须是数字',
        'deadline_of_day.required'    => '您还未填写交图天数',
        'deadline_of_day.integer'     => '交图天数必须是数字',
        'n_cost'                      => '套系可约单数必须大于0',
        'start_time'                  => '请填写开始时间',
        'end_time'                    => '请填写开始时间'
    ];

    /**
     * 额外规则
     * @param array $data
     */
    public function custom_validate($data)
    {
        $messages = [];
        if (empty($data['works'])) {
            $messages[] = "该套系至少应绑定一个样片集";
        }
        if (empty($data['category'])) {
            $messages[] = "该套系必须至少选择一个分类";
        }
        if ($data['modeling_count'] < $data['modeling_count_from_user']) {
            $messages[] = "摄影师最多提供数不可大于造型总数量";
        }
        //整形判断
        if (isset($data["allow_self_take"])) {
            if(empty($data["modeling_count_from_user"])) {
                $messages[] = '您还未填写允许用户自带数量';
            };
            if(!is_integer($data["modeling_count_from_user"])) {
                $messages[] = '自带数量是数字';
            };
        }

        if ($data['start_time'] > $data['end_time']) {
            $messages[] = "套系开始时间必须小于结束时间";
        }

        if ($data["is_promotion"] == 1 && empty($data['promotion']['id'])) {
            if ($data['promotion']['promotion_start_time'] >= $data['promotion']['promotion_end_time']) {
                $messages[] = "特惠开始时间应小于特惠停止时间";
            }
            if ((int)$data['promotion']['per_user_count']  > $data['promotion']['p_order_count']) {
                $messages[] = "特惠的限购额度应小于特惠数量";
            }
            if ($data['promotion']['p_order_count']  <= 0 ) {
                $messages[] = "特惠数量应大于0";
            }

            if ($data['promotion']['promotion_start_time'] < $data['start_time']) {
                $messages[] = "特惠的开始时间应大于套系的起始有效时间";
            }

            if ($data['promotion']['promotion_end_time'] > $data['end_time']) {
                $messages[] = "特惠的结束时间应小于在套系终止有效时间";
            }
            $startdate=strtotime($data['promotion']['promotion_start_time']);
            $enddate=strtotime($data['promotion']['promotion_end_time']);
            $days=round(($enddate-$startdate)/3600/24) ;
            if ($days > 100) {
                $messages[] = "特惠时间长度不应大于100天";
            }
            if ($data['promotion']['promotion_price'] >= $data['price']) {
                $messages[] = "特惠价格应小于套系价格";
            }

            if ($data['promotion']['promotion_price'] < 5) {
                $messages[] = "特惠价格应大于5元";
            }
            $validator = new PromotionValidator();
            $res = $validator->validate($data['promotion']);
            if($res->fails()) {
                $messages[] = $validator->getMessages();
            }
            unset($validator);
        }
        if (isset($data["attachment"]) && is_array($data["attachment"]) && count($data["attachment"]) > 0) {
            $validator = new AttachmentValidator();
            foreach($data["attachment"] as $item) {
                if(!empty($item['title'])) {
                    $res = $validator->validate($item);
                    if($res->fails()) {
                        $messages[] = $validator->getMessages();
                        break;
                    }
                }
            }
            unset($validator);
        }
        self::$result->setCustomMessages($messages);
        return empty($messages);
    }

}