<?php

namespace App\Works\v1\Controller;

use App\Http\Response\HttpStatus;
use App\Http\Response\Response;
use App\Works\Events\WorksEditedEvent;
use App\Works\Model\Works as WorksModel;
use App\Works\Validation\WorksValidator;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Symfony\Component\HttpFoundation\Request;
use Illuminate\Support\Facades\Event;
use App\Works\Model\Details as DetailModel;

class Works extends \Illuminate\Routing\Controller
{
    use \App\Traits\Controller;
    /**
     * @var array
     * 查询筛选字段
     */
    protected $query_fields = [
        "uid","name","is_del","is_hidden"
    ];

    /**
     * 样片列表
     * @method get
     * @param
     * @param  uid   int    option
     * @param  shelf int    option
     * @param
     */
    public function index()
    {
        $query = request()->query();
        $where = $this->filter($query);
        $where = $this->setBlurSearch($where,"name");
        is_null($where) && $where = [];
        $row  = isset($query["size"])?$query["size"]:config("app.default_perpage");
        return Response::success(WorksModel::where($where)->paginate($row)->toArray());
    }

    /**
     * 样片基础数据
     * @param $id
     * @return array
     */
    public function show($id)
    {
        try {
            $info = WorksModel::findOrFail($id);
            $info->getRelate(request("relation"));
            return Response::success($info->toArray());
        } catch (ModelNotFoundException $e){
            return Response::error(HttpStatus::HttpNotFound,"样片未找到");
        }
    }

    /**
     * 获取关联信息
     * @param $id
     * @param $relation
     * @return array
     */
    public function getRelate($id,$relation)
    {
        try {
            $model = WorksModel::findOrFail($id , ["id"]);
            $model->getRelate($relation);
            return Response::success($model);
        } catch (ModelNotFoundException $e){
            return Response::error(HttpStatus::HttpNotFound,"样片未找到");
        }
    }

    /**
     * 新建样片
     * @method post
     */
    public function store(Request $request)
    {
        $validator = new WorksValidator();
        $post = $request->all();
        $res = $validator->validate($post);
        if ($res->fails()) {
            $messages = $validator->getMessages();
            return Response::error($messages,HttpStatus::HttpValidationFailed);
        }
        $model = WorksModel::newWorks($post);
        if ($model !== false) {
            Event::fire(new WorksEditedEvent($model));
            return Response::ok();
        } else {
            return Response::error();
        }
    }

    /**
     * 编辑样片基础信息
     * @param Request $request
     * @method put
     */
    public function update($id,Request $request)
    {
        try {
            $model = WorksModel::findOrFail($id);
            $res = $model->update($request->all());
            if ($res) {
                Event::fire(new WorksEditedEvent($model));
                return Response::ok();
            } else {
                return Response::error();
            }
        } catch (ModelNotFoundException $e) {
            return Response::error(HttpStatus::HttpNotFound,"样片未找到");
        }
    }


    /**
     * 删除一个样片
     * @method delete
     */
    public function destory($id)
    {
        try {
            $model = WorksModel::findOrFail($id);
            $res = $model->delete();
            if ($res) {
                Event::fire(new WorksEditedEvent($model));
                return Response::ok();
            } else {
                return Response::error();
            }
        } catch (ModelNotFoundException $e) {
            return Response::error(HttpStatus::HttpNotFound,"样片未找到");
        }
    }
}